@@ -9,6 +9,9 @@
 #define BIG_WIDTH 390
 #define BIG_HEIGH (BIG_WIDTH * 5398 / 8660)
 
+#define OTHER_WIDTH 550
+#define OTHER_HEIGH (OTHER_WIDTH * 5398 / 8660)
+
 #define MAX_WIDTH 780
 #define MAX_HEIGH (MAX_WIDTH * 5398 / 8660)
 
@@ -24,6 +27,9 @@
 #define BIG_WIDTH 350
 #define BIG_HEIGH (BIG_WIDTH * 5398 / 8660)
 
+#define OTHER_WIDTH 550
+#define OTHER_HEIGH (OTHER_WIDTH * 5398 / 8660)
+
 #define MAX_WIDTH 350
 #define MAX_HEIGH (MAX_WIDTH * 5398 / 8660)
 
@@ -37,13 +43,14 @@
 #endif
 
 // internal image size
-#define INTERNAL_WIDTH 400
-#define INTERNAL_HEIGH (INTERNAL_WIDTH * 5398 / 8660)
+#define _INTERNAL_WIDTH 400
+#define _INTERNAL_HEIGH (_INTERNAL_WIDTH * 5398 / 8660)
+
 
 #define B_SIZE 105
 #define KEY_SIZE 16
 
-#define VERSION "1.2-2"
+#define VERSION "1.3-0"
 
 #define TEXT_ABOUT "<qt><br><br><p><i><b>qtWallet</b></i>, v.1.0</p>"\
 			"<p>Developed by <b>nowheremanmail@gmail.com</b></p>"\
@@ -72,6 +79,7 @@ ID-3 	125 × 88 mm 	Passports and Visas
 #define TYPE_DATE 2
 #define TYPE_STRING 0
 #define TYPE_INTEGER 1
+#define TYPE_IMAGE 7
 
 //#define FOLDER "MyDocs/qtwallet"
 
