@@ -162,17 +162,21 @@ public:
 	bool backupDB(const QString & prefix);
 	bool restoreDB(const QString & prefix);
 	QString changePassword (const QString & newPassword);
+        QString mergeDB (const QString & dbToMerge, const QString & passwordToMerge);
 
 	bool renameGroup (const QString & from, const QString & to);
 	bool deleteGroup (const QString & from);
 
 private:
+        bool _deleteTarget(int idtarget);
 	bool deleteDB(const QString & prefix);
 	bool _openDB(const QString & name, const QString & cName = "");
-	bool initDB();
+        bool _openFileDB(const QString & name, const QString & cName = "");
+        bool initDB();
 	bool isInit();
 
 	bool migrationFrom (DatabaseManager * mgr);
+        bool _mergeDB (DatabaseManager * mergeDB);
 
 	bool checkPassword (QString a, int v);
 	bool createPassword (QString a, int v);
