@@ -20,18 +20,31 @@
 #endif
 #include <QDesktopServices>
 
-//#define CAMERA
 
-#ifdef CAMERA
-#include "camera.h"
-#endif
-
-QLabelW::QLabelW(const QString &text, bool isLabel, QWidget *parent, Qt::WindowFlags f) : QLabel (text, parent, f) {
+QLabelW::QLabelW(const QString &text, bool isLabel, bool isImage, bool isEdit, QWidget *parent, Qt::WindowFlags f) : QLabel (text, parent, f) {
 	this->isLabel = isLabel;
+	this->isImage = isImage;
+	this->isEdit = isEdit;
+	this->modified = false;
+	this->image = NULL;
 
-	if (!isLabel) {
+	if ((!isLabel) || isImage) {
 		setContextMenuPolicy(Qt::CustomContextMenu);
 		QObject::connect(this, SIGNAL(customContextMenuRequested(const QPoint&)), this, SLOT(labelMenu(const QPoint&)));
+
+		if (isImage) {
+			this->setBackgroundRole(QPalette::NoRole);
+			//this->setContentsMargins(5, 2, 0, 2);
+			this->setMaximumSize(OTHER_WIDTH, OTHER_HEIGH);
+			//this->setMinimumSize(BIG_WIDTH, BIG_HEIGH);
+			this->setScaledContents(true);
+//			}
+//			else {
+//				imageLabel->setMaximumSize(MAX_WIDTH, MAX_HEIGH);
+//				imageLabel->setMinimumSize(MAX_WIDTH, MAX_HEIGH);
+//				imageLabel->setScaledContents(true);
+//			}
+		}
 	}
 }
 
@@ -39,12 +52,63 @@ QLabelW::QLabelW(QWidget *parent, Qt::WindowFlags f): QLabel (parent, f) {
 }
 
 QLabelW::~QLabelW() {
+	if (image != NULL) {
+		qDebug () << "destroy image " << image;
+		delete image; // TODO sure??
+		image = NULL;
+	}
 }
 
+QImage * QLabelW::getPicture () {
+	return image;
+}
+
+void QLabelW::readImage (TargetField * field) {
+	if (image == NULL)
+		field->value = "";
+	else {
+		eWallet::preProcessImage (image);
+
+		QByteArray im2;
+		QBuffer buffer2(&im2);
+
+		buffer2.open(QIODevice::WriteOnly);
+		image->save(&buffer2, "PNG"); // writes image into ba in PNG format
+
+		field->value = QString (im2.toBase64());
+	}
+	qDebug () << "getting IMAGE field " << image << " " << field->value.length() << " bytes";
+}
+
+void QLabelW::setPicture (TargetField * field) {
+	//TargetField * f = ((ButtonInfo *)this->userData(0))->field;
+
+	image = NULL;
+	if (field->value.length() > 0) {
+		image = new QImage ();
+
+		qDebug () << "exist image " << field->value.length() << " bytes";
+		//QByteArray im_s;
+		//decrypt(query.value(0).toByteArray(), im_s);
+		if (!image->loadFromData (QByteArray::fromBase64(field->value.toAscii()))) {
+
+		}
+	}
+
+	if (image == NULL) {
+		QPixmap available(":/resources/empty_card.png");
+		this->setPixmap(available);
+	}
+	else {
+		this->setPixmap(QPixmap::fromImage (*image));
+	}
+}
+
+
 void QLabelW::labelMenu(const QPoint & _p) {
    	TargetField * f = ((ButtonInfo *)this->userData(0))->field;
 
-	if (f->value != NULL && f->value.length() > 0) {
+	if ((f->value != NULL && f->value.length() > 0) || f->type == TYPE_IMAGE) {
 		QMenu menu(this);
 		if (f->type == TYPE_URL) {
 			menu.addAction(tr("Go to"), this, SLOT(m_goto()));
@@ -64,21 +128,90 @@ void QLabelW::labelMenu(const QPoint & _p) {
 				menu.addAction(tr("Copy"), this, SLOT(m_copy()));
 			}
 			else {
-				menu.addAction(tr("Copy"), this, SLOT(m_copy()));
+				if (f->type == TYPE_IMAGE) {
+					if (isEdit) {
+						menu.addAction(tr("Edit"), this, SLOT(m_editImage()));
+						//menu.addAction(tr("Mirror"), this, SLOT(mirrorFace()));
+						menu.addAction(tr("Rotate"), this, SLOT(m_rotateImage()));
+						menu.addAction(tr("Delete"), this, SLOT(m_deleteImage()));
+					}
+					else {
+						menu.addAction(tr("Export"), this, SLOT(m_exportImage()));
+					}
+				}
+				else
+					menu.addAction(tr("Copy"), this, SLOT(m_copy()));
 			}
 		}
 		menu.exec(_p);
 	}
 }
+void QLabelW::m_editImage () {
+   	TargetField * f = ((ButtonInfo *)this->userData(0))->field;
+
+	QString path(QDesktopServices::storageLocation(QDesktopServices::PicturesLocation));
+	QString s = QFileDialog::getOpenFileName(
+	                this,
+	                tr("Choose image"),
+	                path,
+	                tr("Images (*.png *.jpg *.gif *.jpeg);;"));
+
+	if (!s.isNull () && !s.isEmpty ()) {
+		////qDebug () << "loading " << s;
+		if (image) delete image;
+
+		image = new QImage ();
+		DetailWindow::loadImage(settings, s, *image); //;
+		////qDebug () << "loaded " << s;
+		this->setPixmap(QPixmap::fromImage (*image));
+		////qDebug () << "asigned " << s;
+		//update();
+		//deleteFace = false;
+		modified = true;
+	}
+}
+void QLabelW::m_rotateImage () {
+   	TargetField * f = ((ButtonInfo *)this->userData(0))->field;
+
+   	if (image != NULL) {
+		QImage * tmp = image;
+
+		QMatrix m;
+		m.rotate(90);
+		image = new QImage (tmp->transformed(m));
+
+		this->setPixmap(QPixmap::fromImage (*image));
+		delete tmp;
+		modified = true;
+   	}
+}
+void QLabelW::m_deleteImage () {
+   	TargetField * f = ((ButtonInfo *)this->userData(0))->field;
+
+   	if (image != NULL) {
+		modified = true;
+		delete image;
+		image = NULL;
+		QPixmap available(":/resources/empty_card.png");
+		this->setPixmap(available);
+   	}
+}
+void QLabelW::m_exportImage () {
+   	TargetField * f = ((ButtonInfo *)this->userData(0))->field;
+
+	DetailWindow::exportImage(this, image, f->name, tr("Choose image file"));
+}
 
 void QLabelW::m_copy () {
    	TargetField * f = ((ButtonInfo *)this->userData(0))->field;
 
+	if (f->type != TYPE_IMAGE) {
 #ifdef Q_WS_MAEMO_5
-	show_banner(this, tr("%1 has been saved to clipboard").arg(f->name));
+		show_banner(this, tr("%1 has been saved to clipboard").arg(f->name));
 #endif
-	QClipboard *cb = QApplication::clipboard();
-	cb->setText(f->value);
+		QClipboard *cb = QApplication::clipboard();
+		cb->setText(f->value);
+	}
 }
 
 void QLabelW::m_goto () {
@@ -535,8 +668,6 @@ DetailWindow::DetailWindow(Target * _target, const QString & groupName, int iste
 	componentLayout = new QGridLayout ();
 #endif
 
-	componentLayout->setColumnMinimumWidth(1, 100);
-
 	showFields ();
 
 	vboxlayout->addLayout(componentLayout);
@@ -571,6 +702,11 @@ void DetailWindow::modifyFields() {
 				field->value = ((QTextEdit*)values[i])->toPlainText();
 				fieldsChanged |= ((QTextEdit*)values[i])->document ()->isModified ();
 			}
+			else
+			if (field->type == TYPE_IMAGE) {
+				fieldsChanged |= ((QLabelW*)values[i])->isModified ();
+				((QLabelW*)values[i])->readImage(field);
+			}
 			else {
 				//qDebug () << "getting LIN field " << i;
 				field->value = ((QLineEdit*)values[i])->text();
@@ -582,6 +718,8 @@ void DetailWindow::modifyFields() {
 		createFieldDlg->setMinimumSize(size().width(),size().height()-100);
 		createFieldDlg->exec();
 
+		// TODO sort fields!!!
+
 		showFields ();
 
 		delete createFieldDlg;
@@ -659,11 +797,11 @@ void DetailWindow::_addField (TargetField * field) {
 		QLineEdit * nameLineEdit = new QLineEdit(this);
 		nameLineEdit->setText (field->value);
 		if (settings.value("hints/phoneNumber/supportCharacters").toBool()) {
-			qDebug () << "phone support characters";
+			//qDebug () << "phone support characters";
 			nameLineEdit->setInputMethodHints(Qt::ImhNoAutoUppercase | Qt::ImhNoPredictiveText);
 		}
 		else {
-			qDebug () << "phone only supports numbers";
+			//qDebug () << "phone only supports numbers";
 			nameLineEdit->setInputMethodHints(Qt::ImhDigitsOnly);
 		}
 		componentLayout->addWidget(nameLineEdit, pos, 1, 1, 1);
@@ -682,6 +820,15 @@ void DetailWindow::_addField (TargetField * field) {
 		nameLineEdit->setFocusPolicy(Qt::StrongFocus);
 		//nameLineEdit->setContentsMargins(0, 0, 20, 0);
 	}
+	else
+	if (field->type == TYPE_IMAGE) {
+		QLabelW * imageLabel = new QLabelW("", false, true, true, this);
+		imageLabel->setUserData(0, new ButtonInfo (field));
+		componentLayout->addWidget(imageLabel, pos, 1, 1, 1);
+		imageLabel->setPicture(field);
+
+		this->values.append (imageLabel);
+	}
 	else {
 		QLineEdit * nameLineEdit = new QLineEdit(this);
 		componentLayout->addWidget(nameLineEdit, pos, 1, 1, 1);
@@ -741,21 +888,21 @@ void DetailWindow::showFields () {
 							for (; i < parent->fieldsByType->at (elType)->size(); i++) {
 								TargetField tmp = parent->fieldsByType->at (elType)->at(i);
 								if (tmp.originalName == field->name) {
-									nameLabel = new QLabelW(tmp.name, true, this);
+									nameLabel = new QLabelW(tmp.name, true, false, false, this);
 									i = -1;
 									break;
 								}
 							}
 							if (i != -1) {
-								nameLabel = new QLabelW(field->name, true, this);
+								nameLabel = new QLabelW(field->name, true, false, false, this);
 							}
 						}
 						else {
-							nameLabel = new QLabelW(field->name, true, this);
+							nameLabel = new QLabelW(field->name, true, false, false, this);
 						}
 					}
 					else {
-						nameLabel = new QLabelW(field->name, true, this);
+						nameLabel = new QLabelW(field->name, true, false, false, this);
 					}
 
 					nameLabel->setFont (QFont("arial", 11, 1, true));
@@ -778,22 +925,24 @@ void DetailWindow::showFields () {
 					}
 					else {
 						if (field->type == TYPE_TEXT) {
-							QLabel * nameLineEdit = new QLabelW(field->value, false, this);
+							QLabel * nameLineEdit = new QLabelW(field->value, false, false, false, this);
 							nameLineEdit->setUserData(0, new ButtonInfo (field));
 							nameLineEdit->setWordWrap(true);
 							componentLayout->addWidget(nameLineEdit, pos, 1, 1, 1);
-							//componentLayout->setRowMinimumHeight(pos, 300);
-							//QObject::connect(nameLineEdit, SIGNAL(customContextMenuRequested(const QPoint&)), this, SLOT(labelMenu(const QPoint&)));
+						}
+						else
+						if (field->type == TYPE_IMAGE) {
+							QLabelW * imageLabel = new QLabelW("", false, true, edit, this);
+							imageLabel->setUserData(0, new ButtonInfo (field));
+
+							componentLayout->addWidget(imageLabel, pos, 1, 1, 1);
+							imageLabel->setPicture(field);
 						}
 						else {
-							QLabel * nameLineEdit = new QLabelW(field->value, false, this);
+							QLabel * nameLineEdit = new QLabelW(field->value, false, false, false, this);
 							nameLineEdit->setUserData(0, new ButtonInfo (field));
 							nameLineEdit->setWordWrap(true);
 							componentLayout->addWidget(nameLineEdit, pos, 1, 1, 1);
-
-
-							//QObject::connect(nameLineEdit, SIGNAL(customContextMenuRequested(const QPoint&)), this, SLOT(labelMenu(const QPoint&)));
-
 						}
 					}
 				}
@@ -915,13 +1064,17 @@ void DetailWindow::saveTarget () {
 
 	for (int i = 0; i < target->fields.count(); i++) {
 		TargetField * field = target->fields[i];
-		//qDebug () << "getting field " << i;
+		qDebug () << "getting field " << i;
 		
 		field->name = labels[i]->text();
 		if (field->type == TYPE_TEXT) {
 			//qDebug () << "getting TEX field " << i;
 			field->value = ((QTextEdit*)values[i])->toPlainText();
 		}
+		else
+		if (field->type == TYPE_IMAGE) {
+			((QLabelW*)values[i])->readImage(field);
+		}
 		else {
 			//qDebug () << "getting LIN field " << i;
 			field->value = ((QLineEdit*)values[i])->text();
@@ -1035,14 +1188,39 @@ void DetailWindow::_newTarget () {
 	close ();
 }
 
-void DetailWindow::loadImage (QString q, QImage& res) {
+
+
+void DetailWindow::loadImage (QSettings & settings, QString q, QImage& res) {
 	QImageReader reader;
 	reader.setFileName(q);
 	//Set/scale image size into screen
-	QSize imageSize = reader.size();
-	QSize ss (INTERNAL_WIDTH, INTERNAL_HEIGH);
-	imageSize.scale(ss, Qt::KeepAspectRatio);
-	reader.setScaledSize(ss);
+//	QSize imageSize = reader.size();
+//
+//	qDebug () << "IMAGE SIZE " << imageSize.width() << " " << imageSize.height();
+//
+//	if (imageSize.height() > settings.value("internalImage/HEIGH", _INTERNAL_HEIGH).toInt() || imageSize.width() > settings.value("internalImage/WIDTH", _INTERNAL_WIDTH).toInt()) {
+//		QSettings settings;
+//		if (settings.value("internalImage/scale", true).toBool()) {
+//			//qDebug () << "scaling image " << ::INTERNAL_WIDTH << " " << ::INTERNAL_HEIGH;
+//			QSize ss (settings.value("internalImage/WIDTH", _INTERNAL_WIDTH).toInt(), settings.value("internalImage/HEIGH", _INTERNAL_HEIGH).toInt());
+//			switch (settings.value("internalImage/aspect", 1).toInt()) {
+//			case 1:
+//				qDebug () << "scaling with keep aspect image";
+//				imageSize.scale(ss, Qt::KeepAspectRatio);
+//				break;
+//			case 2:
+//				qDebug () << "scaling with expanding aspect image";
+//				imageSize.scale(ss, Qt::KeepAspectRatioByExpanding);
+//				break;
+//			default:
+//				imageSize.scale(ss, Qt::IgnoreAspectRatio);
+//			}
+//			qDebug () << "scaled image " << ss.width() << " " << ss.height();
+//			reader.setScaledSize(ss);
+//		}
+//	}
+//	else {
+//	}
 	// Read image
 	res = reader.read();
 	////qDebug () << "read " << q << " as (" << imageSize << ") with result " << reader.errorString ();
@@ -1136,34 +1314,40 @@ void DetailWindow::deleteCardFace()
 
 	deleteFace = true;
 }
-void DetailWindow::exportCardFace()
+
+void DetailWindow::exportImage(QWidget * witdget, QImage * picture, QString name, QString ti)
 {
-	if (target->picture != NULL) {
+	if (picture != NULL) {
 		QString path(QDesktopServices::storageLocation(QDesktopServices::PicturesLocation));
 
-		path.append("/").append(target->name.simplified().replace(' ', '_')).append(".png");
+		path.append("/").append(name.simplified().replace(' ', '_')).append(".png");
 
 		QString s = QFileDialog::getSaveFileName(
-	                this,
-	                tr("Choose card face image file"),
+	                witdget,
+	                ti,
 	                path,
 	                tr("Images (*.png);;"));
 		if (!s.isNull () && !s.isEmpty ()) {
 			if (s.endsWith(".jpg"))
-				target->picture->save(s, "JPG"); // writes image into ba in PNG format
+				picture->save(s, "JPG"); // writes image into ba in PNG format
 			else if (s.endsWith(".jpeg"))
-				target->picture->save(s, "JPEG"); // writes image into ba in PNG format
+				picture->save(s, "JPEG"); // writes image into ba in PNG format
 			else if (s.endsWith(".png"))
-				target->picture->save(s, "PNG"); // writes image into ba in PNG format
+				picture->save(s, "PNG"); // writes image into ba in PNG format
 			else if (s.endsWith(".tiff"))
-				target->picture->save(s, "TIFF"); // writes image into ba in PNG format
+				picture->save(s, "TIFF"); // writes image into ba in PNG format
 			else {
 				s.append(".png");
-				target->picture->save(s, "PNG"); // writes image into ba in PNG format
+				picture->save(s, "PNG"); // writes image into ba in PNG format
 			}
 		}
 	}
 }
+
+void DetailWindow::exportCardFace()
+{
+	exportImage(this, target->picture, target->name, tr("Choose card face image file"));
+}
 void DetailWindow::cardFace()
 {
 	QString path(QDesktopServices::storageLocation(QDesktopServices::PicturesLocation));
@@ -1178,7 +1362,7 @@ void DetailWindow::cardFace()
 		if (currentImageN) delete currentImageN;
 
 		currentImageN = new QImage ();
-		loadImage(s, *currentImageN); //;
+		loadImage(settings, s, *currentImageN); //;
 		////qDebug () << "loaded " << s;
 		imageLabel->setPixmap(QPixmap::fromImage (*currentImageN));
 		////qDebug () << "asigned " << s;
@@ -1277,6 +1461,20 @@ void DetailWindow::cardFaceMenu(const QPoint & pos) {
 	}
 	menu.exec(pos);
 }
+void DetailWindow::imageMenu(const QPoint & pos) {
+	QMenu menu(this);
+	if (edit) {
+		menu.addAction(tr("Edit"), this, SLOT(cardFace()));
+		//menu.addAction(tr("Mirror"), this, SLOT(mirrorFace()));
+		menu.addAction(tr("Rotate"), this, SLOT(rotateFace()));
+		menu.addAction(tr("Delete"), this, SLOT(deleteCardFace()));
+	}
+	else {
+		menu.addAction(tr("Export Face"), this, SLOT(exportCardFace()));
+	}
+	menu.exec(pos);
+}
+
 void DetailWindow::cardBackMenu(const QPoint & pos) {
 	QMenu menu(this);
 	if (edit) {
@@ -1295,8 +1493,8 @@ void DetailWindow::cardBackMenu(const QPoint & pos) {
 void DetailWindow::orientationChanged(int o) {
 	////qDebug () << "CHAGNED";
 
+	QRect screenGeometry = QApplication::desktop()->screenGeometry();
 	if (!edit) {
-		QRect screenGeometry = QApplication::desktop()->screenGeometry();
 		if (screenGeometry.width() > screenGeometry.height()) {
 			if (imageLabel != NULL) {
 				imageLabel->setMaximumSize(MAX_WIDTH, MAX_HEIGH);
@@ -1318,6 +1516,17 @@ void DetailWindow::orientationChanged(int o) {
 			}
 		}
 	}
+	else {
+	}
+
+	if (screenGeometry.width() > screenGeometry.height()) {
+		componentLayout->setColumnMinimumWidth(0, 180);
+		componentLayout->setColumnMinimumWidth(1, 580);
+	}
+	else {
+		componentLayout->setColumnMinimumWidth(0, 80);
+		componentLayout->setColumnMinimumWidth(1, 360);
+	}
 }
 
 
@@ -1330,31 +1539,7 @@ void DetailWindow::deleteCardBack()
 }
 void DetailWindow::exportCardBack()
 {
-	if (target->pictureBack != NULL) {
-		QString path(QDesktopServices::storageLocation(QDesktopServices::PicturesLocation));
-
-		path.append("/").append(target->name.simplified().replace(' ', '_')).append("_BACK.png");
-
-		QString s = QFileDialog::getSaveFileName(
-	                this,
-	                tr("Choose card back image file"),
-	                path,
-	                tr("Images (*.png);;"));
-		if (!s.isNull () && !s.isEmpty ()) {
-			if (s.endsWith(".jpg"))
-				target->pictureBack->save(s, "JPG"); // writes image into ba in PNG format
-			else if (s.endsWith(".jpeg"))
-				target->pictureBack->save(s, "JPEG"); // writes image into ba in PNG format
-			else if (s.endsWith(".png"))
-				target->pictureBack->save(s, "PNG"); // writes image into ba in PNG format
-			else if (s.endsWith(".tiff"))
-				target->pictureBack->save(s, "TIFF"); // writes image into ba in PNG format
-			else {
-				s.append(".png");
-				target->pictureBack->save(s, "PNG"); // writes image into ba in PNG format
-			}
-		}
-	}
+	exportImage(this, target->pictureBack, target->name, tr("Choose card back image file")); // _BACK
 }
 
 void DetailWindow::cardBack()
@@ -1370,7 +1555,7 @@ void DetailWindow::cardBack()
 		if (currentImageBackN) delete currentImageBackN;
 		
 		currentImageBackN = new QImage ();
-		loadImage(s, *currentImageBackN); //;
+		loadImage(settings, s, *currentImageBackN); //;
 		
 		imageBackLabel->setPixmap(QPixmap::fromImage (*currentImageBackN));
 		//update ();
@@ -1407,6 +1592,11 @@ void DetailWindow::closeEvent(QCloseEvent *event) {
 				//qDebug () << "getting TEX field " << i;
 				isModified |= ((QTextEdit*)values[i])->document ()->isModified ();
 			}
+			else
+			if (field->type == TYPE_IMAGE) {
+				//qDebug () << "getting TEX field " << i;
+				isModified |= ((QLabelW*)values[i])->isModified ();
+			}
 			else {
 				//qDebug () << "getting LIN field " << i;
 				isModified |= ((QLineEdit*)values[i])->isModified();
@@ -1550,6 +1740,7 @@ CreateFieldDialog::CreateFieldDialog(Target * target, DetailWindow *paTarget) :
 	typeLineEdit->addItem (tr("Phone Number"), QVariant(TYPE_PHONE_NUMBER));
 	typeLineEdit->addItem (tr("Password"), QVariant(TYPE_PASSWORD));
 	typeLineEdit->addItem (tr("URL"), QVariant(TYPE_URL));
+	typeLineEdit->addItem (tr("Image"), QVariant(TYPE_IMAGE));
 
 	componentLayout->addWidget(typeLineEdit, 2, 3, 1, 4);
 
