@@ -55,7 +55,7 @@ public:
 	~DetailWindow ();
 	
 	void addField (TargetField * field);
-	static void loadImage (QString q, QImage& res);
+	static void loadImage (QSettings & settings, QString q, QImage& res);
 
 private:
 	int pos;
@@ -126,10 +126,13 @@ public slots:
 	void deleteCardBack();
 	void exportCardBack();
 
+	static void exportImage(QWidget * witdget, QImage * im, QString d, QString tit);
+
 //	void cardMenu(const QPoint & pos);
 //	void labelMenu(const QPoint & pos);
 	void cardFaceMenu(const QPoint & pos);
 	void cardBackMenu(const QPoint & pos);
+	void imageMenu(const QPoint & pos);
 //	void contextMenuEvent (QContextMenuEvent * event);
 
 	void emitCommitData (int);
@@ -161,18 +164,40 @@ public:
 class QLabelW: public QLabel {
 	Q_OBJECT
 public:
-	QLabelW(const QString &text, bool isLabel, QWidget *parent=0, Qt::WindowFlags f=0);
+	QLabelW(const QString &text, bool isLabel, bool isImage, bool isEdit, QWidget *parent=0, Qt::WindowFlags f=0);
 	QLabelW(QWidget *parent=0, Qt::WindowFlags f=0);
 	~QLabelW();
+
+	inline bool isModified () {
+		return this->modified;
+	}
+
 public slots:
+	void readImage (TargetField * field);
 	void labelMenu(const QPoint & _p);
 	void m_copy ();
 	void m_goto ();
 	void m_sms ();
 	void m_call ();
+
+	void m_editImage ();
+	void m_rotateImage ();
+	void m_deleteImage ();
+	void m_exportImage ();
+
+	QImage * getPicture ();
+	void setPicture (TargetField * field);
+
 protected:
 	bool isLabel;
     void mousePressEvent(QMouseEvent *);
+
+    QImage * image;
+    bool isImage;
+    bool isEdit;
+    bool modified;
+	QSettings settings;
+
 };
 
 #endif // DETAIL_H
