@@ -39,9 +39,7 @@ public:
 
 	void resetF ();
 	void keyPressEvent(QKeyEvent* event);
-#ifdef Q_WS_MAEMO_5
 	void orientationChanged(int o);
-#endif
 	void focus ();
 
 private slots:
