@@ -7,7 +7,6 @@
  * TODO after change type it is not refreshed on list
  * TODO implements copy of information field (and images?)
  * TODO check types on import
- * TODO sort after insert new card
  * TODO pc version?
  * TODO fix problems with key
  * TODO after import groups are not refresh
@@ -15,6 +14,10 @@
  * TODO import from xml
  * TODO move fields value into a big field
  * TODO what happens with small pictures
+ * TODO add contacts
+ * TODO plugin to allow link some OS info inside Wallet
+ * TODO widget to link some info on desktop
+ * TODO zoom on pictures!!
   */
 
 #include "utils.h"
@@ -148,6 +151,9 @@ eWallet::eWallet(QWidget *parent) : QMainWindow(parent)
 
     setWindowTitle("Wallet");
 
+//    INTERNAL_WIDTH = settings.value("internalImage/WIDTH", _INTERNAL_WIDTH).toInt();
+//    INTERNAL_HEIGH = settings.value("internalImage/HEIGH", _INTERNAL_HEIGH).toInt();
+
 	isFirst = true;
 	currentGroup = NULL;
 	centralWidget = NULL;
@@ -1013,6 +1019,7 @@ void eWallet::createMenu()
 		//menuBar()->addAction(tr("Change View"), this, SLOT(changeMode()));
 		menuBar()->addAction(tr("Change Password"), this, SLOT(changePassword()));
 		menuBar()->addAction(tr("About"), this, SLOT(about()));
+		menuBar()->addAction(tr("Options"), this, SLOT(options()));
 		menuBar()->addAction(tr("Exit"), QApplication::instance(), SLOT(quit()));
 	}
 #else
@@ -1037,10 +1044,12 @@ void eWallet::createMenu()
 		m->addAction(tr("Import"), this, SLOT(import()));
 		m->addAction(tr("Export"), this, SLOT(exportData()));
 		m->addAction(tr("Export Xml"), this, SLOT(exportXml()));
-                m->addSeparator();
-                m->addAction(tr("Merge DB"), this, SLOT(mergeDB()));
-                m->addSeparator();
-                menuBar()->addAction(tr("Exit"), QApplication::instance(), SLOT(quit()));
+		m->addSeparator();
+		m->addAction(tr("Merge DB"), this, SLOT(mergeDB()));
+		m->addSeparator();
+		m->addAction(tr("Options"), this, SLOT(options()));
+		m->addSeparator();
+		menuBar()->addAction(tr("Exit"), QApplication::instance(), SLOT(quit()));
 		menuBar()->addAction(tr("About"), this, SLOT(about()));
 		//menuBar()->addAction(tr("Change View"), this, SLOT(changeMode()));
 	}
@@ -1055,6 +1064,9 @@ void eWallet::sort()
 
 bool eWallet::__insertTarget(Target* target)
 {
+	preProcessImage (target->picture);
+	preProcessImage (target->pictureBack);
+
 	if (::dbManager->insertTarget(target)) {
 		////qDebug () << "Adding new item";
 		::dbManager->updateFields (target);
@@ -1207,6 +1219,8 @@ bool eWallet::_updateTarget(Target* target)
 bool eWallet::_updateTargetPicture(Target* target)
 {
 	////qDebug () << "UPDATE ON " << i;
+	preProcessImage (target->picture);
+
 	if (::dbManager->updateTargetPicture(target)) {
 		if (isShowGroups == 2) {
 			int i = list.indexOf(target);
@@ -1219,6 +1233,8 @@ bool eWallet::_updateTargetPicture(Target* target)
 }
 bool eWallet::_updateTargetPictureBack(Target* target)
 {
+	preProcessImage (target->pictureBack);
+
 	if (::dbManager->updateTargetPictureBack(target)) {
 		return true;
 	}
@@ -1375,6 +1391,9 @@ void eWallet::import() {
 								  case 'H':
 									  type = TYPE_PHONE_NUMBER;
 									  break;
+								  case 'I':
+									  type = TYPE_IMAGE;
+									  break;
 								  case 'S':
 								  default:
 					    			  type = TYPE_STRING;
@@ -1414,6 +1433,7 @@ void eWallet::import() {
 //							  else
 								  field->type = TYPE_STRING;
 				    	  }
+				    	  // TODO TYPE_IMAGE
 						  field->value = line.mid(j + 1).trimmed();
 						  field->order = 10 * (i - 3);
 						  field->encrypted = true;
@@ -1444,7 +1464,7 @@ void eWallet::import() {
 						QFile imgFile (imgPath);
 						if (imgFile.exists()) {
 							QImage * currentImageN = new QImage ();
-							DetailWindow::loadImage(imgPath, *currentImageN); //;
+							DetailWindow::loadImage(settings, imgPath, *currentImageN); //;
 							target->setPicture(currentImageN);
 
 							::dbManager->updateTargetPicture (target);
@@ -1461,7 +1481,7 @@ void eWallet::import() {
 						QFile imgFile (imgPath);
 						if (imgFile.exists()) {
 							QImage * currentImageN = new QImage ();
-							DetailWindow::loadImage(imgPath, *currentImageN); //;
+							DetailWindow::loadImage(settings, imgPath, *currentImageN); //;
 							target->setPictureBack(currentImageN);
 
 							::dbManager->updateTargetPictureBack (target);
@@ -1664,6 +1684,9 @@ void eWallet::exportData() {
 						  case TYPE_PHONE_NUMBER:
 								t << field->name << ";H: " << field->value << "\n";
 							  break;
+						  case TYPE_IMAGE:
+								t << field->name << ";I: " << field->value << "\n"; // TODO
+							  break;
 						  case 'S':
 						  default:
 								t << field->name << ": " << field->value << "\n";
@@ -1810,6 +1833,9 @@ void eWallet::exportXml() {
 						  case TYPE_PHONE_NUMBER:
 								t << "<field id=\"" << field->id << "\" name=\"" << FIX_VALUE(field->name) << "\" type=\"H\">" << FIX_VALUE(field->value) << "</field>\n";
 							  break;
+						  case TYPE_IMAGE:
+								t << "<field id=\"" << field->id << "\" name=\"" << FIX_VALUE(field->name) << "\" type=\"I\">" << FIX_VALUE(field->value) << "</field>\n";
+							  break;
 						  case 'S':
 						  default:
 								t << "<field id=\"" << field->id << "\" name=\"" << FIX_VALUE(field->name) << "\">" << FIX_VALUE(field->value) << "</field>\n";
@@ -1997,6 +2023,13 @@ void eWallet::askPassword () {
 	loginDialogDlg->focus ();
 }
 
+void eWallet::options() {
+	optionsDlg = new OptionsDialog(this);
+	optionsDlg->setMinimumSize(size().width(),size().height()-100);
+	optionsDlg->exec();
+
+}
+
 void eWallet::about() {
 	//this->setStyleSheet("QMessageBox#A { background-color: yellow }");
 	QMessageBox::about(this, tr("About Wallet"), tr("<qt><br><br><p><i><b>qtWallet</b></i>, v.1.0</p>"\
@@ -2015,6 +2048,43 @@ bool eWallet::finalPassword (const QString & pass) {
 	return false;
 }
 
+void eWallet::preProcessImage (QImage * image) {
+	if (image == NULL) return;
+
+	QSettings settings;
+
+	int h = settings.value("internalImage/HEIGH", _INTERNAL_HEIGH).toInt();
+	int w = settings.value("internalImage/WIDTH", _INTERNAL_WIDTH).toInt();
+
+	//QImage res QImage::scaled (const QSize & size, Qt::AspectRatioMode aspectRatioMode = Qt::IgnoreAspectRatio, Qt::TransformationMode transformMode = Qt::FastTransformation ) const
+	//QSize imageSize = reader.size();
+	qDebug () << "IMAGE SIZE " << image->width() << " " << image->height();
+
+	if (image->height() > h || image->width() > w) {
+		if (settings.value("internalImage/scale", true).toBool()) {
+			QImage res;
+			switch (settings.value("internalImage/aspect", 1).toInt()) {
+			case 1:
+				qDebug () << "scaling with keep aspect image";
+				res = image->scaled(w, h, Qt::KeepAspectRatio, Qt::SmoothTransformation);
+				break;
+			case 2:
+				qDebug () << "scaling with expanding aspect image";
+				res = image->scaled(w, h, Qt::KeepAspectRatioByExpanding, Qt::SmoothTransformation);
+				break;
+			default:
+				res = image->scaled(w, h, Qt::IgnoreAspectRatio, Qt::SmoothTransformation);
+			}
+
+			*image = res;
+		}
+	}
+	else {
+		// TODO
+	}
+}
+
+
 void eWallet::changePassword () {
 	times = 0;
 	isChangingPassword = true;
@@ -2110,3 +2180,105 @@ void eWallet::reallyChangePassword (const QString & newPassword) {
 }
 
 
+OptionsDialog::OptionsDialog(QWidget *parent): QDialog(parent) {
+	setWindowTitle("Wallet");
+
+	setContextMenuPolicy(Qt::NoContextMenu);
+	QHBoxLayout * layout = new QHBoxLayout(this);
+
+	layout->setMargin(0);
+	layout->setSpacing(0);
+
+	layout->setSpacing(10);
+	layout->setMargin(10);
+
+	QDialogButtonBox* dlgBtnBox = new QDialogButtonBox(this);
+	dlgBtnBox->setOrientation(Qt::Vertical);
+
+//	QPushButton * closeButton = new QPushButton(tr("Save && Close"), this);
+//	//closeButton->setFixedHeight(70);
+//	closeButton->setContextMenuPolicy(Qt::NoContextMenu);
+//	closeButton->setFocusPolicy(Qt::NoFocus);
+//	closeButton->setIconSize(QSize(48, 48));
+//	QObject::connect(closeButton, SIGNAL(clicked()), this, SLOT(save()));
+//	dlgBtnBox->addButton(closeButton, QDialogButtonBox::ActionRole);
+
+	QGridLayout * componentLayout = new QGridLayout ();
+	componentLayout->setContentsMargins(15, 0, 15, 0);
+	componentLayout->setSpacing(5);
+
+
+	phoneChars = new QCheckBox(tr("Phone fields with characters"), this);
+	phoneChars->setContextMenuPolicy(Qt::NoContextMenu);
+	phoneChars->setChecked(settings.value("hints/phoneNumber/supportCharacters", false).toBool());
+	QObject::connect(phoneChars, SIGNAL(clicked()), this, SLOT(changePhoneChars()));
+	componentLayout->addWidget(phoneChars, 0, 1, 1, 1);
+
+	// Name label
+//	nameLabel = new QLabel(, this);
+//	nameLabel->setContextMenuPolicy(Qt::NoContextMenu);
+//	//nameLabel->setAlignment(Qt::AlignTop);
+//	componentLayout->addWidget(nameLabel, 2, 1, 2, 2);
+
+	fixImageSize = new QCheckBox(tr("Limit image size"), this);
+	fixImageSize->setContextMenuPolicy(Qt::NoContextMenu);
+	fixImageSize->setChecked(settings.value("internalImage/scale", true).toBool());
+	QObject::connect(fixImageSize, SIGNAL(clicked()), this, SLOT(changeFix()));
+	componentLayout->addWidget(fixImageSize, 0, 2, 1, 1);
+
+	QLabel * nameLabel;
+
+	nameLabel = new QLabel(tr("Max image width"), this);
+	nameLabel->setContextMenuPolicy(Qt::NoContextMenu);
+	componentLayout->addWidget(nameLabel, 1, 1, 1, 1);
+
+	maxWidth = new QSpinBox (this);
+	maxWidth->setMaximum(1024);
+	maxWidth->setMinimum(1);
+	maxWidth->setValue(settings.value("internalImage/WIDTH", _INTERNAL_WIDTH).toInt());
+	QObject::connect(maxWidth, SIGNAL(valueChanged(int)), this, SLOT(valueChangedW(int)));
+	componentLayout->addWidget(maxWidth, 1, 2, 1, 2);
+
+	nameLabel = new QLabel(tr("Max image height"), this);
+	nameLabel->setContextMenuPolicy(Qt::NoContextMenu);
+	componentLayout->addWidget(nameLabel, 2, 1, 1, 1);
+
+	maxHeigh = new QSpinBox (this);
+	maxHeigh->setMaximum(1024);
+	maxHeigh->setMinimum(1);
+	maxHeigh->setValue(settings.value("internalImage/HEIGH", _INTERNAL_HEIGH).toInt());
+	QObject::connect(maxHeigh, SIGNAL(valueChanged(int)), this, SLOT(valueChangedH(int)));
+	componentLayout->addWidget(maxHeigh, 2, 2, 1, 2);
+
+	maxWidth->setEnabled(fixImageSize->isChecked());
+	maxHeigh->setEnabled(fixImageSize->isChecked());
+
+	layout->addLayout(componentLayout);
+	layout->setAlignment(componentLayout, Qt::AlignVCenter);
+
+	layout->addWidget(dlgBtnBox);
+	layout->setAlignment(dlgBtnBox, Qt::AlignBottom);
+
+	setLayout(layout);
+}
+
+OptionsDialog::~OptionsDialog() {
+
+}
+void OptionsDialog::changeFix () {
+	settings.setValue("internalImage/scale", fixImageSize->isChecked());
+
+	maxWidth->setEnabled(fixImageSize->isChecked());
+	maxHeigh->setEnabled(fixImageSize->isChecked());
+}
+void OptionsDialog::changePhoneChars () {
+	settings.setValue("hints/phoneNumber/supportCharacters", phoneChars->isChecked());
+}
+void OptionsDialog::valueChangedW (int i) {
+	settings.setValue("internalImage/WIDTH", i);
+
+	maxHeigh->setValue(i * 5398 / 8660);
+}
+void OptionsDialog::valueChangedH (int i) {
+	settings.setValue("internalImage/HEIGH", i);
+}
