@@ -18,6 +18,31 @@
 
 extern DatabaseManager * dbManager;
 
+class OptionsDialog: public QDialog {
+	Q_OBJECT
+
+public:
+	OptionsDialog(QWidget *parent = 0);
+	~OptionsDialog();
+
+private:
+	QSettings settings;
+	QCheckBox * fixImageSize;
+	QCheckBox * phoneChars;
+
+	QSpinBox * maxWidth;
+	QSpinBox * maxHeigh;
+
+	QComboBox * typeImageSize;
+
+private slots:
+	void changeFix ();
+	void changePhoneChars ();
+	void valueChangedW (int i);
+	void valueChangedH (int i);
+};
+
+
 class RenameGroupDialog: public QDialog {
 	Q_OBJECT
 
@@ -40,10 +65,13 @@ class eWallet: public QMainWindow
 	Q_OBJECT
 
 public:
+
 	eWallet (QWidget * parent = 0);
 	eWallet (QString * group, eWallet *_parent, QWidget *parent = 0);
 	~eWallet ();
 
+	static void preProcessImage (QImage * image);
+
 	bool finalPassword (const QString & pas);
 
 	bool _deleteTarget(Target* target);
@@ -110,16 +138,20 @@ private slots:
 	void _cancelSearch ();
 	void _hideSearch ();
 	void about ();
+	void options ();
 	void import();
 	void exportData();
 	void exportXml();
-        void mergeDB();
-        void textChanged (const QString &);
+	void mergeDB();
+	void textChanged (const QString &);
 	void contextMenuEvent (QContextMenuEvent * event);
 	void renameGroup ();
 	void deleteGroup ();
 	
 private:
+	QSettings settings;
+	QPointer<OptionsDialog> optionsDlg;
+
 	void askPassword ();
 	void closeEvent(QCloseEvent *event);
 	bool isFirst;
