@@ -3,6 +3,7 @@
  */
 
 #include "eWallet.h"
+#include "Constants.h"
 
 #include <QtGui>
 #include <QApplication>
@@ -13,6 +14,28 @@ int main(int argc, char *argv[])
 {
 //	QCoreApplication::setAttribute(Qt::AA_NativeWindows);
 
+
+//	QString path(QDir::home().path());
+//	path.append(QDir::separator()).append(FOLDER);
+//
+//	QDir folder (path);
+//	if (!folder.exists()) {
+//		qDebug () << "creating folder " << path;
+//		folder.mkdir(path);
+//
+//		path.append(QDir::separator()).append("qtwallet.db.sqlite");
+//		path = QDir::toNativeSeparators(path);
+//
+//		QString pathold(QDir::home().path());
+//		pathold.append(QDir::separator()).append("qtwallet.db.sqlite");
+//		pathold = QDir::toNativeSeparators(pathold);
+//
+//		QFile::rename(pathold, path);
+//
+//		qDebug () << "move from " << pathold << " to " << path;
+//	}
+
+
     QApplication app(argc, argv);
 
     QString lang;
