@@ -1,9 +1,10 @@
 #ifndef UTILS_H
 #define UTILS
 
-#include <gtk/gtk.h>
-#include <hildon/hildon.h>
-#include <hildon/hildon-notification.h>
+
+//#include <gtk/gtk.h>
+//#include <hildon/hildon.h>
+//#include <hildon/hildon-notification.h>
 
 
 #include <Qt>
